import { ServiceConfig as ServiceConfig$1 } from '@cleavelandprice/ngx-lib/authentication';
import * as i0 from '@angular/core';
import { EventEmitter, Injectable, Optional, Output, NgModule, SkipSelf } from '@angular/core';
import * as i2 from '@angular/common/http';
import { CommonModule } from '@angular/common';

//import { ServiceConfig as NgxLibServiceConfig } from '@cleavelandprice/ngx-lib';
class ServiceConfig extends ServiceConfig$1 {
    // See the comment in the library's main service-config.ts file for an explanation
    constructor(useSsl_or_customApiUrl = false, appName = undefined, tokenName = 'user', authenticateEndpoint = '/authenticate') {
        super(useSsl_or_customApiUrl, appName, tokenName, authenticateEndpoint);
    }
}

class AuthenticationService {
    //#endregion
    //#region Properties
    get useSessionStorage() {
        return !!localStorage.getItem('storeTokenInSession');
    }
    get storage() {
        return this.useSessionStorage ?
            sessionStorage :
            localStorage;
    }
    //#endregion
    //#region Lifecycle
    constructor(config, http) {
        this.config = config;
        this.http = http;
        //#region Fields
        this.authenticatedChanged = new EventEmitter();
        this.authenticatingChanged = new EventEmitter();
        this.authenticationError = new EventEmitter();
        this.authenticating = false; // Flag set to true while the service is attempting to authenticate the user
        this.authenticated = false; // Flag set to true when the user is successfully authenticated
        this.config = this.config ?? new ServiceConfig();
        this.checkForExistingToken();
    }
    //#endregion
    //#region Utilities
    //Authenticate the user and store the token in Local Storage
    login(username, password, adminGroup, additionalInfo) {
        const request = {
            username,
            password,
            adminGroup,
            additionalInfo,
            appName: this.config.appName
        };
        this.authenticating = true;
        this.authenticatingChanged.emit(this.authenticating);
        this.http.post(`${this.config.apiUrl}${this.config.authenticateEndpoint}`, request)
            .subscribe({
            next: (user) => {
                if (user) {
                    this.validateUser(user);
                }
                else {
                    this.logout();
                }
            },
            error: (err) => {
                this.authenticating = false;
                this.authenticatingChanged.emit(this.authenticating);
                this.authenticationError.emit(err);
            }
        });
    }
    //Logout the user and delete the token from Local Storage
    logout() {
        if (this.storage.getItem(this.config.tokenName)) {
            this.storage.removeItem(this.config.tokenName);
        }
        this.authenticatedUser = undefined;
        this.authenticated = false;
        this.authenticatedChanged.emit(false);
    }
    validateUser(token) {
        this.authenticatedUser = {
            admin: false,
            displayName: token.displayName,
            accountName: token.sAMAccountName,
            distinguishedName: token.distinguishedName,
            employeeNumber: token.employeeNumber,
            memberOf: token.memberOf.join(',')
        };
        this.storage.setItem(this.config.tokenName, JSON.stringify(token));
        this.authenticated = true;
        this.authenticatedChanged.emit(this.authenticated);
        this.authenticating = false;
        this.authenticatingChanged.emit(this.authenticating);
    }
    checkForExistingToken() {
        const token = this.storage.getItem(this.config.tokenName);
        if (token) {
            this.validateUser(JSON.parse(token));
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.4", ngImport: i0, type: AuthenticationService, deps: [{ token: ServiceConfig, optional: true }, { token: i2.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "16.2.4", ngImport: i0, type: AuthenticationService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.4", ngImport: i0, type: AuthenticationService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: ServiceConfig, decorators: [{
                    type: Optional
                }] }, { type: i2.HttpClient }]; }, propDecorators: { authenticatedChanged: [{
                type: Output
            }], authenticatingChanged: [{
                type: Output
            }], authenticationError: [{
                type: Output
            }] } });

class ObjectModule {
    static forRoot(config) {
        return {
            ngModule: ObjectModule,
            providers: [
                AuthenticationService,
                { provide: ServiceConfig, useValue: config }
            ]
        };
    }
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('ObjectModule is already loaded. Import it in the AppModule only');
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.4", ngImport: i0, type: ObjectModule, deps: [{ token: ObjectModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.2.4", ngImport: i0, type: ObjectModule, imports: [CommonModule] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.2.4", ngImport: i0, type: ObjectModule, imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.4", ngImport: i0, type: ObjectModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [
                        CommonModule
                    ],
                    exports: []
                }]
        }], ctorParameters: function () { return [{ type: ObjectModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }]; } });

/**
 * Generated bundle index. Do not edit.
 */

export { AuthenticationService, ObjectModule, ServiceConfig };
//# sourceMappingURL=cleavelandprice-ngx-lib-authentication-object.mjs.map
